@@ -228,10 +228,6 @@ class DSLibrary(object):
         open_args.pop("format", None)
         open_args.pop("format_options", None)
         open_args.pop("dask", None)
-        storage_options = open_args.pop("storage_options", None)
-        if storage_options:
-            # for dask read methods we pass these as 'storage_options', everywhere else they are open_args
-            open_args.update(storage_options)
         # send all outputs to mlflow if requested
         use_mlflow = self._mlflow_all and ("w" in mode or "a" in mode)
         if use_mlflow:
@@ -563,6 +559,9 @@ class DSLibrary(object):
             format_options      Options compatible with pandas.to_{format}()
             sql_table           Causes dataframe-like data to be stored into a SQL table.
             nosql_collection    Causes dataframe-like data to be stored into a NoSQL table (not yet implemented).
+            dask                True to write a folder instead of a single file, using dask or the conventions of dask.
+
+        :returns:       In the specific case where dask is True, returns the list of generated files.
         """
         use_mlflow = self._mlflow_all
         # map resource name using supplied specs
@@ -571,7 +570,7 @@ class DSLibrary(object):
         if not file_extension:
             self._default_format_for_aliases(resource_name, kwargs)
         format = kwargs.pop("format", None) or file_extension
-        format_options = dict(kwargs.get("format_options") or {})
+        format_options = dict(kwargs.pop("format_options", None) or {})
         # detect SQL target
         if kwargs.get("sql_table"):
             # write data to an SQL table
@@ -593,6 +592,19 @@ class DSLibrary(object):
         # send (pandas) DataFrame as CSV/etc.
         df_format = format if format in PANDAS_WRITE_FORMATS else "csv"
         if hasattr(content, PANDAS_WRITE_FORMATS[df_format]):
+            # dataframes
+            # - the 'dask' flag requests dask-like writes to a folder instead of a single file
+            is_dask = hasattr(content, "dask")
+            use_dask = kwargs.pop("dask", None)
+            if use_dask and not is_dask:
+                # pandas > dask
+                content = dask.dataframe.from_pandas(content, npartitions=1)
+                is_dask = True
+            elif is_dask and not use_dask and df_format not in ("csv", "tab", "tsf"):
+                # dask > pandas
+                content = content.compute()
+                is_dask = False
+            # get write method to call
             df_method = getattr(content, PANDAS_WRITE_FORMATS[df_format])
             # cancel append mode if file does not exist
             if append:
@@ -624,7 +636,24 @@ class DSLibrary(object):
                 format_options["index"] = False
             if format == "json":
                 format_options.update({"orient": "records", "lines": True})
-            with opener(uri, mode) as f_w:
+
+            if use_dask:
+                # send the URI directly to Dask so that it will write out a folder with each chunk
+                # FIXME opener() may do quite a bit more than simply open a file, so this is bound to be error-prone
+                return df_method(uri, **format_options, **kwargs)
+            if is_dask:
+                # in this case we have a dask dataframe but want to send it to a file, not a folder
+                #  - dask only supports this for CSV files
+                format_options["single_file"] = True
+                with tempfile.NamedTemporaryFile(mode='rb', suffix=".csv") as f_tmp:
+                    # write to a unified local CSV file
+                    df_method(f_tmp.name, **format_options)
+                    # use opener() to write to the intended destination
+                    with open(f_tmp.name, mode='rb') as f_r:
+                        with opener(uri, 'wb', **kwargs) as f_w:
+                            shutil.copyfileobj(f_r, f_w)
+                return
+            with opener(uri, mode, **kwargs) as f_w:
                 df_method(f_w, **format_options)
             return
         # save matplotlib figures as images
@@ -670,7 +699,7 @@ class DSLibrary(object):
                     json.dump(content, f_w)
                     f_w.write("\n")
         else:
-            raise Exception(f"Unsupported content type: {type(content)}")
+            raise Exception(f"Unsupported content type or format: {type(content)}")
 
     def _write_file(self, opener, resource_name: str, filename: str, append: bool=False):
         """
@@ -749,7 +778,7 @@ class DSLibrary(object):
                         content = self._in_memory_append(prev, content)
                 self._spec["capture"][resource_name] = content
                 return
-            self._write_content(self.open_resource, resource_name, content=content, append=append, **kwargs)
+            return self._write_content(self.open_resource, resource_name, content=content, append=append, **kwargs)
         elif from_filename is not None:
             use_mlflow = self._mlflow_all
             if use_mlflow:
@@ -828,7 +857,7 @@ class DSLibrary(object):
                 sql = open_args.pop("sql", None)
                 table = open_args.pop("sql_table", None)
                 if table:
-                    sql = f"SELECT * from {sql_enquote_id(table)}"
+                    sql = f"SELECT * from {sql_enquote_id(table, dotted=True)}"
                 db_conn = self.get_sql_connection(resource_name, **open_args)
                 try:
                     c = db_conn.cursor()
@@ -884,8 +913,11 @@ class DSLibrary(object):
             sample_size = open_args.pop("sample_size", None) or 30000
             if sample_size < 0:
                 return
-            with use_opener(uri, mode='rb', **open_args) as f_r:
-                return f_r.read(sample_size)
+            try:
+                with use_opener(uri, mode='rb', **open_args) as f_r:
+                    return f_r.read(sample_size)
+            except IsADirectoryError:
+                pass
         def sizer():
             with use_opener(resource_name, mode='rb') as f_r:
                 if not hasattr(f_r, "seek") or not f_r.seekable():
@@ -924,7 +956,10 @@ class DSLibrary(object):
         if read_method:
             if use_dask:
                 storage_options = open_args.get("storage_options") or open_args
-                return read_method(uri, storage_options=storage_options, **read_args)
+                try:
+                    return read_method(uri, storage_options=storage_options, **read_args)
+                except IsADirectoryError:
+                    return read_method(uri + "/*", storage_options=storage_options, **read_args)
             with use_opener(uri, mode='rb', **open_args) as f_r:
                 if format == "xlsx" and not f_r.seekable():
                     # the XLSX reader requires its stream to be seekable so we download to a temporary file
