@@ -151,6 +151,42 @@ def _set_conn_read_only(conn):
         print(f"couldn't set read-only mode on connection: {conn.__class__.__name__}")
 
 
+def dataframe_to_sql(dataframe, table_name: str, append: bool=False):
+    """
+    Generate SQL statements to create a table and add data.
+    :returns a generator of (sql, parameter list)
+    """
+    def enquote(name):
+        name = str(name)
+        if re.search(r'[^A-Za-z0-9_]', name) or name[:1].isdigit():
+            return f'"{name}"'
+        return name
+    if not append:
+        yield "DROP TABLE IF EXISTS %s" % enquote(table_name), []
+    dtype_to_sql = {"int64": "INTEGER", "float64": "DOUBLE"}
+    cols = list(dataframe.columns)
+    types = []
+    for col, dtype in zip(cols, dataframe.dtypes):
+        sql_type = dtype_to_sql.get(str(dtype)) or f"VARCHAR({dataframe[col].str.len().max()+3})"
+        types.append(sql_type)
+    rows = dataframe.itertuples(index=False, name=None)
+    fields = ", ".join("%s %s" % (enquote(col), t) for col, t in zip(cols, types))
+    yield "CREATE TABLE IF NOT EXISTS %s (%s)" % (enquote(table_name), fields), []
+    ins0 = "INSERT INTO %s (%s) VALUES " % (enquote(table_name), ", ".join(map(enquote, cols)))
+    sqls = []
+    params = []
+    for row in (rows or []):
+        sql = "(%s)" % ", ".join(map(lambda v: "%s", row))
+        sqls.append(sql)
+        params += list(row)
+        if len(sqls) >= 1000:
+            yield "%s%s" % (ins0, ", ".join(sqls)), params
+            sqls.clear()
+            params.clear()
+    if sqls:
+        yield "%s%s" % (ins0, ", ".join(sqls)), params
+
+
 METHODS = {
     "pymysql": connect_pymysql,
     "psycopg2": connect_psycopg2,
