@@ -66,7 +66,7 @@ class TestToMMLibrary(unittest.TestCase):
             def get_binary_from_resource(self, resource_name):
                 return b'a\tb\n1\t2\n3\t4\n'
         dsl = DSLibraryViaMMLibrary(_mm=MyMM())
-        df = dsl.load_dataframe("x.csv", format_options={"sep": "\t"})
+        df = dsl.load_dataframe("x.csv", format_options={"delimiter": "\t"})
         assert list(df.columns) == ["a", "b"]
         assert list(df.a) == [1, 3]
 
